
/*
 * FORFILES.C - apply DOS command to list of files
 *  Written by Scott Gourley
 *  (Generalized for compilation under MS C and Borland C by LZ)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>

enum {
    ERR_OK, ERR_NO_LIST_FILE, ERR_NO_COMMAND,
    ERR_LIST_FILE_NOT_FOUND, 
    ERR_LISTED_FILE_NOT_FOUND
};

char *errString[] =
{
 "Success",
 "No list file parameter given. \nSyntax:  "
         "FORFILES  <listfile> <command> <cmd-args>",
 "No command parameter given.  \nSyntax:  "
         "FORFILES  <listfile> <command> <cmd-args>",
 "Specified list file not found.",
 "File specified in list not found.",
};

#define BUFSIZE 80
      
void Error(int err)
{
 printf ("\n\n%s\n", errString[err]);
 exit (err);
}

void Warning(char *str, int err)
{
 printf ("%s:  %s\n", str, errString[err]);
}


void Help(void)
{
 char *lines[] =
 {
 "FORFILES\n\n",
 "This program executes an external command (.BAT .COM or .EXE)\n",
 "for each file listed in an ASCII text file.\n",
 "\n",
 "The text file should only contain one \"8.3\" filename per line,\n",
 "with no spaces or other extraneous characters on the line, and\n",
 "no comment lines, blank lines, etc.\n\n",
 "The specified command executes with its first parameter as the\n",
 "filename from the list, and additional parameters as specified\n",
 "from the FORFILES command line.  To reorganize these parameters,\n",
 "use an intervening batch file.\n"
 "\n",
 "The syntax of the FORFILES program is:\n\n",
 "\tFORFILES <listfile> <command> <params>\n",
 "\n",
 "where:\n",
 "\t<listfile> is the name of the ASCII text list of filenames\n",
 "\t<command> is the name of the program or batch file to execute\n",
 "\t<params> are the additional parameters for the command\n\n",
 };
 int numlines = sizeof(lines) / sizeof(lines[0]);
 int i;

 for (i = 0 ; i < numlines ; i++)
  printf (lines[i]);

 exit (ERR_NO_LIST_FILE);
}

void RunForFile(char *runargs[])
{
 FILE *forfile;
 int i;
  

 /* check for existence of file */

 forfile = fopen(runargs[1], "r");
 if (!forfile)
 {
  Warning(runargs[1], ERR_LISTED_FILE_NOT_FOUND);
  return;
 }
 else
  fclose (forfile);
 
 printf ("%s %s ...\n", runargs[0], runargs[1]);
 
 /* run the command based on the listed file name */

 if (spawnvp(P_WAIT, runargs[0], runargs) == -1)
 {
  printf ("Spawn error:  %d\n", errno);
  exit (255);
 }
}

void RunForFiles(FILE *listfile, int argc, char *argv[])
{
 char buf[BUFSIZE];
 char *runargs[30];
 char *error;
 int i, j;
 

 /* fill in all invariant arguments for spawn */

 runargs[0] = argv[2];
 
 for (i = 3, j = 2; i < argc && j < 30 ; i++, j++)
 {
  runargs[j] = argv[i];
 }
 
 runargs[j] = NULL;
  
 /* get each listed file name and act on it */

 error = fgets(buf, BUFSIZE, listfile);
 while (error)
 {
  buf[strlen(buf)-1] = '\0';
      
  runargs[1] = buf;

  RunForFile(runargs);
      
  error = fgets(buf, BUFSIZE, listfile);
 }
 
}

void main (int argc, char *argv[])
{
 FILE *listfile;
 
 if (argc == 1)
  Error(ERR_NO_LIST_FILE);
  
 if (!strcmp (argv[1], "?"))
  Help();

 if (argc == 2)
  Error(ERR_NO_COMMAND);
  
 listfile = fopen(argv[1], "r");
 if (!listfile)
  Error(ERR_LIST_FILE_NOT_FOUND);
  
 RunForFiles(listfile, argc, argv);
 
 fclose(listfile);
 
 Error(ERR_OK);
}
